!-------------------------------------------------------------------------
!
! Procedures for handling of a dynamical list  
!
!  InitList - initialisation of a list
!  AddList  - add a next item to the list
!  GetImage  - return a pointer to an array of an item
!  PutImage  - return a pointer to an array of an item
!  GetPar    - return the 'skymod,skysig' for this item
!  PutPar    - write the 'skymod,skysig' for this item
!  
!-------------------------------------------------------------------------

Module LibList

! Item type:
 
       type ImageType
           Integer :: i
           Real :: skymod,skysig
           Real,  pointer :: image(:,:)
           type( ImageType ), pointer :: next
           type( ImageType ), pointer :: prev
       end type 

type( ImageType ), private, pointer :: list, current, previous, first, last

contains !---------------------------------------------------------------

Subroutine InitList ( nx, ny, array, istat )

! Initialize a new list
! 
! Input: 
!       nx, ny .. dimension of an image 
!
! Output:
!
!        array .. created image
!        istat .. status

Integer :: nx,ny,istat
real, pointer :: array(:,:)

   if( istat /= 0 ) return
   istat = 10
  
   nullify( list )
   Allocate( list )
   nullify( list%next )
   list%i = 1
   Allocate( list%image(nx,ny) )
!   if( .not. associated(list%image) ) return 

   first => list
   current => list 
   last => current
   array => current%image   

   istat = 0

end subroutine

!-------------------------------------------------------------------------

Subroutine AddItem ( nx, ny, array, istat )

! Initialize a new list
! 
! Input: 
!       nx, ny .. dimension of an image 
!
! Output:
!
!        array .. created image
!        istat .. status

Integer :: nx,ny,istat
real, pointer :: array(:,:)

   if( istat /= 0 ) return

   istat = 10

   Allocate( current%next )
   nullify( current%next%next )
   current%next%i = current%i + 1
   Allocate( current%next%image(nx,ny) )
!   if( .not. associated(current%next%image) ) return
   current => current%next

   array => current%image
   istat = 0

end subroutine

! ------------------------------------------------------------------------


Subroutine GetImage( n, array, istat )

! Get image from list
!
! Input:
!       n ... Number of image
!
! Output:
!      array .. image

Integer :: n, istat
real, pointer :: array(:,:)

   if( istat /= 0 ) return

istat = 11

      current => first
      do while ( associated( current ))
        if( current%i == n ) then
          array => current%image
          istat = 0
          return
        endif
        current => current%next
      enddo

end subroutine

!----------------------------------------------------------------------

Subroutine PutImage( n, array, istat )

! Put image to list
!
! Input:
!       n ... Number of image
!       array .. image
!

Integer :: n, istat
real, pointer :: array(:,:)

   if( istat /= 0 ) return

istat = 11

      current => first
      do while ( associated( current ))
        if( current%i == n ) then
          current%image => array
          istat = 0
          return
        endif
        current => current%next
      enddo

end subroutine


! ----------------------------------------------------------------------

Subroutine GetPar ( n, skymod, skysig, istat )

! Get parameters 
!
! Input:
!       n ... Number of image
!
! Output:
!      skymod, skysig .. parameters

Integer :: n, istat
Real :: skymod, skysig

if( istat /= 0 ) return

istat = 20

      current => first
      do while ( associated( current ))
        if( current%i == n ) then
          skymod = current%skymod
          skysig = current%skysig
          istat = 0
          return
        endif
        current => current%next
      enddo

end subroutine

! ------------------------------------------------------------------------

Subroutine PutPar ( n, skymod, skysig, istat )

! Put parameters 
!
! Input:
!       n ... Number of image
!
! Output:
!      skymod, skysig .. parameters

Integer :: n, istat
Real :: skymod, skysig

if( istat /= 0 ) return

istat = 21
      current => first
      do while ( associated( current ))
        if( current%i == n ) then
          current%skymod = skymod
          current%skysig = skysig
          istat = 0
          return
        endif
        current => current%next
      enddo

end subroutine

!-----------------------------------------------------------------------

end module



