/*  
     PICKO     Convert utility Buil's PIC  -> FITS   
*/

/* FITS I/O related procedures */ 

#include <stdio.h>
#include <string.h>
#include "picko.h"

static char FTSbuf[FITSSIZE];
static int nbuf;

/* ------------------------------------------------------------------------ */

FILE *FITScreate(name,mode)
char *name,*mode;
{
FILE *fs;
int i;

if( (fs = fopen(name,mode)) == NULL ) return(NULL);

nbuf = 0;
for( i = 0; i < FITSSIZE; i ++ )
  FTSbuf[i] = ' ';

return(fs);

}

/* ------------------------------------------------------------------------- */

int FITShead(f,h,parname,tcor)
FILE *f;
HEAD *h;
char *parname;
long tcor;
{
int i,status;
char buf[81];
char date[9],time[9];
int FTSrec(),FTSkyj(),FTSkyc(),FTSkye(),FTScom(),timdat();
FILE *fp;

date[0] = h->comment1[3]; date[1] = h->comment1[4];
date[2] = '/';
date[3] = h->comment1[0]; date[4] = h->comment1[1];
date[5] = '/';
date[6] = h->comment1[6]; date[7] = h->comment1[7];
date[8] = '\0';

time[0] = h->comment1[11]; time[1] = h->comment1[12];
time[2] = ':';
time[3] = h->comment1[14]; time[4] = h->comment1[15];
time[5] = ':';
time[6] = h->comment1[17]; time[7] = h->comment1[18];
time[8] = '\0';

if( strlen(date) > 0 && strlen(time) > 0 )
  timdat(date,time,tcor);
else  
  printf("Warning: date or time not found.\n");

status = 0;

FTSrec(f,"SIMPLE  =                    T / file does conform to FITS standard",status);
FTSrec(f,"BITPIX  =                   16 / number of bits per data pixel",status);
FTSrec(f,"NAXIS   =                    2 / number of data axes",status);
FTSkyj(f,"NAXIS1",h->nrows,"/ length of data axis 1",status);
FTSkyj(f,"NAXIS2",h->nlines,"/ length of data axis 2",status);
FTSkyc(f,"DATE-OBS",date,"/ date",status);
FTSkyc(f,"TIME",time,"/ time, assumed UT",status);
FTSkyj(f,"EXPTIME",h->integration,"/ exposure in seconds",status);
FTSkye(f,"BSCALE",1.0,"/ data scale",status);
FTSkye(f,"BZERO",0.0,"/ data zero",status);
FTSkyc(f,"ORIGIN","PICKO","",status);
FTSkyj(f,"CAMERA",h->camera,"/ Buil's camera identification",status);
FTSkye(f,"HIGHLEV",h->hilev + h->hilev2/1e4,"/ High level",status);
FTSkye(f,"LOWLEV",h->lolev + h->hilev2/1e4,"/ Low level",status);
FTSkyj(f,"X1",h->x1,"/ X1 coordinate",status);
FTSkyj(f,"Y1",h->y1,"/ Y1 coordinate",status);
FTSkyj(f,"X2",h->x2,"/ X2 coordinate",status);
FTSkyj(f,"Y2",h->y2,"/ Y2 coordinate",status);
FTSkyj(f,"BINX",h->binX,"/ binning X",status);
FTSkyj(f,"BINY",h->binY,"/ binning Y",status);
FTScom(f,h->comment1,status);
FTScom(f,h->comment2,status);
strcpy(buf,"Image name : ");
FTScom(f,strcat(buf,h->imagename),status);
strcpy(buf,"Name of associated image : ");
FTScom(f,strcat(buf,h->asocname),status);

if( strlen(parname) > 0 ) {                        /* parameter file */
  if( (fp = fopen(parname,"r")) == NULL ) {
     printf("Can't open parameter file.\n");
     strcpy(parname,"");
     goto parmiss;
  }
  while( fgets(buf,80,fp) != NULL ) {
    for( i = 0; i < strlen(buf); i++ )
       if( buf[i] == '\n' ) 
         buf[i] = ' ';   
    FTSrec(f,buf,status);
  }
  fclose(fp);
}
parmiss:

/*FTScom(f,"File created by the PICKO software from Masaryk University in Brno",status);*/
FTScom(f,VER,status);
FTSrec(f,"END",status);

if( status != 0 ) return(-17);

for( i = 0; i < FITSSIZE; i++ )
   if( FTSbuf[i] == '\0' ) FTSbuf[i] = ' ';

if( fwrite(FTSbuf,1,FITSSIZE,f) != FITSSIZE ) return(-15);

nbuf = 0;
for( i = 0; i < FITSSIZE; i++ )
  FTSbuf[i] = ' ';

return(0);
}

/* --------------------------------------------------------------------- */

int putFITSrow(fs,buf,n)
FILE *fs;
char buf[];
int n;
{
int i,j;
char temp;


for( i = 0; i < 2*n; i+=2 ) {
  temp = buf[i];              /* swap bytes */
  buf[i] = buf[i+1];
  buf[i+1] = temp;
}
for( i = 0; i < n/2; i++ ) { /* flipping */
  temp = buf[2*i];
  buf[2*i] = buf[2*(n - i) - 2];
  buf[2*(n - i) - 2] = temp;
  temp = buf[2*i + 1];
  buf[2*i + 1] = buf[2*(n - i) - 1];
  buf[2*(n - i) - 1] = temp;
}


for( i = 0; i < 2*n; i++) {
  FTSbuf[nbuf] = buf[i];    /* output to buffer */
  nbuf++;
  if( nbuf == FITSSIZE ){  /* buffer full */
    if( fwrite(FTSbuf,1,FITSSIZE,fs) != FITSSIZE )
      return(-1);
    nbuf = 0;
    for( j = 0; j < FITSSIZE; j++)
      FTSbuf[j] = 0;
  } 
} 

return(n);

}

/* ----------------------------------------------------------------------- */

int FITSclose(fs)
FILE *fs;
{
 int i; 
 
 i = fwrite(FTSbuf,1,FITSSIZE,fs);
 fclose(fs);
 return(i);
}

/* ------------------------------------------------------------------------ */

int FTSout(fs)
FILE *fs;
{
int i,j;

if( nbuf == FITSSIZE ) {
  for( i = 0; i < FITSSIZE; i++ )
     if( FTSbuf[i] == '\0' ) FTSbuf[i] = ' ';
  if( fwrite(FTSbuf,1,FITSSIZE,fs) != FITSSIZE )
    return(-1);
  nbuf = 0;
  for( j = 0; j < FITSSIZE; j++)
    FTSbuf[j] = ' ';
}
return(0);
}

/* ----------------------------------------------------------------------- */

int FTSrec(f,st,status)
FILE *f;
char *st;
int status;
{
if( status ) return(status);
strncpy(FTSbuf + nbuf,st,80); 
nbuf = nbuf + 80;
return(FTSout(f));

}

/* ----------------------------------------------------------------------- */

int FTScom(f,st,status)
FILE *f;
char *st;
int status;
{
char buf[81];

if( status ) return(status);

strcpy(buf,"COMMENT ");
strncpy(FTSbuf + nbuf,strcat(buf,st),80);
nbuf = nbuf + 80;
return(FTSout(f));
}

/* ------------------------------------------------------------------------- */

int FTSkyc(f,key,txt,com,status)
FILE *f;
char *key,*txt,*com;
int status;
{
int len;
char buf[81],buf1[21];

if( status ) return(status);

if( (len = strlen(txt)) < 8 )
  sprintf(buf1,"'%-8s'",txt);
else if( len > 20 )
  sprintf(buf1,"'%-20.20s'",txt);
else
  sprintf(buf1,"'%-s'",txt);
sprintf(buf,"%-8.8s= %-20.20s %s",key,buf1,com);
strncpy(FTSbuf + nbuf,buf,80);
nbuf = nbuf + 80;
return(FTSout(f));
}

/* ---------------------------------------------------------------------- */

int FTSkyj(f,key,val,com,status)
FILE *f;
int val,status;
char *com,*key;
{
char buf[81];

if( status ) return(status);

sprintf(buf,"%-8.8s= %20d %s",key,val,com);
strncpy(FTSbuf + nbuf,buf,80);
nbuf = nbuf + 80;
return(FTSout(f));
}

/* -------------------------------------------------------------------- */

int FTSkye(f,key,val,com,status)
FILE *f;
float val;
char *com,*key;
int status;
{
char buf[81];

if( status ) return(status);

sprintf(buf,"%-8.8s= %20.10e %s",key,val,com);
strncpy(FTSbuf + nbuf,buf,80);
nbuf = nbuf + 80;
return(FTSout(f));
}

/* ------------------------------------------------------------------ */

int timdat(date,time,cor)                           /* correction of time */
char *date,*time;
long cor;
{
long tim;  /* time in seconds of the day */
int y,m,d;
int h,min,s,jdat();
double jd,yd,md,dd,datjd();

/* warning !!!!! */
/* the date format (year = 99 or 1999) is unknown for me !!!!! 
 * I'm traing 1999 :-)
 */ 

sscanf(date,"%2d%*1s%2d%*1s%4d",&d,&m,&y);
sscanf(time,"%2d%*1s%2d%*1s%2d",&h,&min,&s);

/* printf("'%s' %d %d %d \n",date,y,m,d); */

tim = s + 60L*(min + 60L*h) + cor;

/* end of 20's century and old data file format*/
if( 50 < y  && y <= 99 )
  yd = y + 1900.0;
else if( 0 <= y && y < 50 )
  yd = y + 2000.0;
else
  yd = y;
      
jd = datjd( yd,(double) m, (double) d);
jd = jd + tim/86400.0;  
jdat(jd,&yd,&md,&dd);
y = yd + 0.5;
m = md + 0.5;
d = dd;
tim = (dd - d)*86400.0 + 0.5;        /* "+ 0.5" for round */
h = tim / 3600L;
min = (tim - 3600L*h) / 60L;
s = tim - 60L*(min + 60L*h);

if( y < 2000 ) 
  sprintf(date,"%02d/%02d/%02d",d,m,y-1900);
else
  sprintf(date,"%04d-%02d-%02d",y,d,m);

sprintf(time,"%02d:%02d:%02d",h,min,s);

return(1);
}

