/*  
     PICKO     Convert utility Buil's PIC  -> FITS   
*/

/*******  I/(O) procedures for Buil's PIC image format files *****/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "picko.h"

#define LENHEAD sizeof(HEAD)

static int ftype;   /* file type: value 0 = uncompressed */ 

FILE *PICopen( name, mode )
  char *name;
  char *mode;
{

return( fopen(name,mode) );

}

HEAD *PIChead(f)
FILE *f;
{
HEAD *h;
unsigned char *buf;

if( (h = malloc(LENHEAD + 1)) == NULL ) return(NULL);

if( (buf = malloc(HEADSIZE + 1)) == NULL ) return(NULL);
 
if( fread(buf,1,HEADSIZE,f) != HEADSIZE ) {
  free(buf);
  free(h);
  return(NULL);
}

h->signature  = *(buf); 
h->version    = *(buf + 1);
h->headsize   = *(buf + 2) + 256* *(buf + 3);

if( h->signature == 0xFC )
  ftype = 1;
else if( h->signature == 0xFB )
  ftype = 0;
else
  ftype = -1;

if( ftype < 0 ) {
  free(buf);
  free(h);
  printf("File type unknown.\n");
  return(NULL);
}

if( h->version != 0x31 )
  printf("Warning: uncorrect file type version: %o.\n",h->version); 
  
strncpy(h->imagename,buf + 4,32);
strncpy(h->asocname,buf + 36,32);
h->nrows      = *(buf + 68) + 256* *(buf + 69);
h->nlines     = *(buf + 70) + 256* *(buf + 71);
h->dataformat = *(buf + 72) + 256* *(buf + 73);
h->datatype   = *(buf + 74) + 256* *(buf + 75);
h->camera     = *(buf + 78) + 256* *(buf + 79);
h->x1         = *(buf + 80) + 256* *(buf + 81);
h->y1         = *(buf + 82) + 256* *(buf + 83); 
h->x2         = *(buf + 84) + 256* *(buf + 85);
h->y2         = *(buf + 86) + 256* *(buf + 87);
h->binX       = *(buf + 88) + 256* *(buf + 89);
h->binY       = *(buf + 90) + 256* *(buf + 91);
h->plans      = *(buf + 92) + 256* *(buf + 93);
strncpy(h->date,buf + 94,10);
strncpy(h->time,buf + 104,14);
h->hilev      = *(buf + 118) + 256* *(buf + 119);
h->lolev      = *(buf + 120) + 256* *(buf + 121);
h->hilev2     = *(buf + 122) + 256* *(buf + 123); 
h->lolev2     = *(buf + 124) + 256* *(buf + 125);
h->integration= *(buf + 126) + 256*( *(buf + 127) 
         + 256*(*(buf + 128) + 256*  *(buf + 129)));
strncpy(h->comment1,buf + 130,80);
strncpy(h->comment2,buf + 210,80);
 
/*
  printf("signature = %x\n",h->signature);
  printf("version = %x\n",h->version);
  printf("Header size= %d\n",h->headsize);
  printf("Image name: %s\n",h->imagename);
  printf("Name of associated image: %s\n",h->asocname);
  printf("Number of rows= %d\n",h->nrows);
  printf("Number of lines= %d\n",h->nlines);
  printf("Data format= %d     Data type= %d\n",h->dataformat,h->datatype);
  printf("Camera= %d\n",h->camera);
  printf("(x1,y1)= (%d,%d)      (x2,y2)=(%d,%d)\n",
                            h->x1,h->y1,h->x2,h->y2); 
  printf("Bining X= %d     Bining Y=%d\n",h->binX,h->binY);
  printf("Color planes= %d\n",h->plans);
  printf("Date= %s\n",h->date);
  printf("Time= %s\n",h->time);
  printf("hilev= %d.%d       lolev = %d.%d\n",
          h->hilev,h->hilev2,h->lolev,h->lolev2);
  printf("Integration  time= %d\n",h->integration);
  printf("%s\n",h->comment1);
  printf("%s\n",h->comment2); 
*/

free(buf);

return(h);
}

int getPICline( fp, buf, nrow )
  FILE *fp;
  int nrow;
  unsigned char *buf;
{
 

if( ftype )           /* uncompressed image */

  return( fread(buf,1,2*nrow,fp)/2 );

else {                     /* compressed image */

  return(0);
}

}  

PIClose(f,h)
FILE *f;
HEAD *h;
{
fclose(f);
if( h != NULL ) free(h);
}
