/*  
     PICKO     Convert utility Buil's PIC  -> FITS   
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "picko.h"

#define MAXLINE    256     /* length of text line */
#define MAXFORMAT   10

#define KNAME "kout????.fts"


int main ( argc, argv )
int argc;
char *argv[];
{
int convpic(),i,n,icit,icit0,stdinput = 0, head = 0;
long tcor;
char dirname[MAXLINE] = "", parname[MAXLINE] = "", *j;
char line[MAXLINE],buf1[MAXLINE],mask[MAXLINE] = KNAME;
FILE *fopen(), *fp;
static char format[MAXFORMAT][5] = 
 { "","%01d", "%02d", "%03d", "%04d", "%05d", "%06d", "%07d", "%08d", "%09d" };

icit = 1; icit0 = 1;
tcor = 0L;

printf("%s\n\n",VERSION);

for( i = 0; i < argc; i++ ) {              /* loop over all parameters */

  if( (j = strstr(argv[i],"@")) != NULL )                   /* dir file */
    if( strlen(argv[i]) == 1 )
      stdinput = 1;
    else
      strncpy(dirname,j + strlen("@"), MAXLINE);

  if( (j = strstr(argv[i],"par=")) != NULL )          /* parameter file */
    strncpy(parname,j + strlen("par="),MAXLINE);

  if( strcmp(argv[i],"-p") == 0 )                  
    strncpy(parname,argv[++i],MAXLINE);
  
  if( (j = strstr(argv[i],"tcor=")) != NULL )        /* time correction */
    sscanf(j + strlen("tcor="),"%ld",&tcor);

  if( (j = strstr(argv[i],"mask=")) != NULL )   /* mask of output files */
    strncpy(mask,j + strlen("mask="),MAXLINE);

  if( (j = strstr(argv[i],"-o")) != NULL )  
    strncpy(mask,argv[++i],MAXLINE);

  if( strcmp(argv[i],"-c") == 0 )                            /* counter */
    sscanf(argv[++i],"%d",&icit0);

  if( strcmp(argv[i],"-h") == 0 )                         /* only header */
    head = 1;

  if( strcmp(argv[i],"-g") == 0 )
    if( (fp = fopen("picko.par","a")) != NULL ) {
      fprintf(fp,"OBJECT  = 'Star'\n");
      fprintf(fp,"OBSERVER= 'J.Walker'\n");
      fprintf(fp,"TELESCOP= '1.0m'\n");
      fprintf(fp,"RA      =            90.0\n");
      fprintf(fp,"DEC     =            90.0\n");
      fprintf(fp,"EQUINOX =          2000.0\n");
      fclose(fp);
      printf("'picko.par' is an example of the parameter file.\n");
      return (0);
    }

  if( strcmp(argv[i],"-L") == 0 ) {            /* license */
    printf("%s\n%s\n%s\n%s\n\n%s\n%s\n%s\n%s\n\n%s\n%s\n%s\n\n",
"    This program is free software; you can redistribute it and/or modify",
"    it under the terms of the GNU General Public License as published by",
"    the Free Software Foundation; either version 2, or (at your option)",
"    any later version.",
"    This program is distributed in the hope that it will be useful",
"    but WITHOUT ANY WARRANTY; without even the implied warranty of",
"    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the",
"    GNU General Public License for more details.",
"    You should have received a copy of the GNU General Public License",
"    along with this program; if not, write to the Free Software",
"    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.");
    return (0);
  }

  if( argc == 1 || strcmp(argv[i],"-help") == 0 ) {
    printf("PICKO        Convert utility Buil's PIC ->  FITS\n\n");
    printf("Usage:picko [options] [@dirfile] [par=parfile] [tcor=?] [mask=?] [image1] ...\n\n");
    printf("@dirfile= file with name(s) of image(s) to convert, no wildcards *,?\n");
    printf("          (for only '@', read image(s) name(s) from standart input)\n");
    printf("image1...= name(s) of simple image(s) to convert, no wildcards *,?\n");
    printf("par= parameter file\n");
    printf("tcor= correction of time in seconds\n");
    printf("mask= output mask (default = kout????.fts)\n");
    printf("options:\n");
    printf("\t-p <parname>\t dark frame image (equivalent with 'par=')\n");
    printf("\t-o <mask>\t output mask (equivalent with 'mask=')\n");
    printf("\t-c <number>\t counter initial value (default = 1)\n");
    printf("\t-h\t  create only header of FITS file\n");
    printf("\t-help\t give this help\n");
    printf("\t-L\t display software license\n");
    printf("Give '-g' switch for an example of parameter file.\n");
    return (0);
  }

}

if( (j = strstr(mask,"*")) != NULL )                /* remove '*' wildcard */
  *j = '?';
if( (j = strstr(mask,"?")) != NULL )                      /* number of '?' */
  for( n = 0; *j == '?' && n < MAXFORMAT-1; j++ )
     n = n + 1;
j = strstr(mask,"?");

icit = icit0;

                                  /* images names read from command line */
for( i = 1; i < argc; i++)
  if( strstr(argv[i],"=") == NULL && strstr(argv[i],"@") == NULL 
  &&  strstr(argv[i],"-") == NULL && strstr(argv[i-1],"-") == NULL ) {
    if( j != NULL ) {
      sprintf(buf1,format[n],icit);
      memcpy(j,buf1,n);
    }
    if( convpic(argv[i],parname,tcor,mask,head) < 0 )
      printf("Can't convert file: %s\n",argv[i]);
    icit = icit + 1; 
  }

if( strlen(dirname) > 0 ) {                   /* images names read from file */
  if( (fp = fopen(dirname,"r")) == NULL ) {
    printf("Can't open file %s\n",dirname);
    exit(0); 
  }
  while( fgets(buf1,MAXLINE,fp) != NULL ) {
    sscanf(buf1,"%s",line);
    if( j != NULL ) {
      sprintf(buf1,format[n],icit);
      memcpy(j,buf1,n); 
    }
    if( convpic(line,parname,tcor,mask,head) < 0 )
      printf("Can't convert file %s \n",line);
    icit = icit + 1;
  }
  fclose(fp);
}
                                      /* images names from standart input */
if( stdinput )
  while( fgets(buf1,MAXLINE,stdin) != NULL ) {
    sscanf(buf1,"%s",line);
    if( j != NULL ) {
      sprintf(buf1,format[n],icit);
      memcpy(j,buf1,n);
    }
    if( convpic(line,parname,tcor,mask,head) < 0 )
      printf("Can't convert file: %s\n",line);
    icit = icit + 1; 
  }

if( argc > 1  && icit == icit0 )
  printf("No input files.\n");

return 0;
}



/************************* Conversion ********************************/

int convpic(name,parname,tcor,ftsname,head)
  char *name,*parname,*ftsname;
  long tcor;
  int head;
{
  FILE *PICopen(),*FITScreate(),*fp,*fs;
  HEAD *PIChead(),*hd;
  int n;
  int getSTrow(),putFITSrow(),FITSclose(),FITShead(),getPICline();
  int endpos(),identify();
  unsigned char *buff;

hd = NULL;
buff = NULL;

printf("%s -> %s\n",name,ftsname);

if( (fp = PICopen(name,"rb")) == NULL ) {
  printf("Can't open the input file.\n");
  return(-1);
}
   
if( (hd = PIChead(fp)) == NULL ) {
  printf("Can't read head of the input image.\n");
  return(-2);
}

/* create FITS file */
if( (fs = FITScreate(ftsname,"wb")) == NULL ) {
  printf("Can't create FITS file.\n");
  fclose(fp);
  return(-11);
}

/* create FITS header */
if( FITShead(fs,hd,parname,tcor) != 0 ) {
  printf("Can't write FITS header.\n");
   fclose(fp);
   fclose(fs);
   return(-12);
} 

if( head ) goto konec;                     /* do not convert image data */

if( (buff = malloc(2*hd->nrows + 1)) == NULL ) {
  printf("Can't create internal buffer.\n");
  goto konec; 
}

while( (n = getPICline(fp,buff,hd->nrows)) != 0 ) {
       if( n == -1 ) {
         printf("Can't read data from the PIC file.\n");
         goto konec; 
       }
       if( putFITSrow(fs,buff,hd->nrows) != hd->nrows ) {
         printf("Can't write data to the FITS file.\n");
         goto konec; 
       }
 }
       
konec:

/* close files and return */
  if( buff != NULL ) free(buff);
  if( FITSclose(fs) != FITSSIZE ) {
    printf("Can't close the FITS file.\n");
    return(-1);
  }
  PIClose(fp,hd);
  return(1);
}


